#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['yaqd_core',
 'yaqd_core._hardware',
 'yaqd_core.aserial',
 'yaqd_core.avrorpc',
 'yaqd_core.exceptions',
 'yaqd_core.logging']

package_data = \
{'': ['*']}

install_requires = \
['appdirs', 'toml', 'fastavro']

extras_require = \
{'dev': ['black', 'pre-commit']}

setup(name='yaqd-core',
      version='2020.7.1',
      description='Core python package for implementing yaq deamons, and associated utilities.',
      author='yaq developers',
      author_email='git@ksunden.space',
      url='https://yaq.fyi',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
